@@ -1,24 +1,25 @@
 """
-This module includes auxiliar functions that are intended to be used only by 
+This module includes auxiliar functions that are intended to be used only by
 the library.
 """
+
 def listType(l):
     """
-    If the type of every element of the list l is the same, this function 
+    If the type of every element of the list l is the same, this function
     returns that type, else it returns None. If l is not a list, this function
     will raise a ValueError.
     """
-    if type(l) is not list:
+    if not isinstance(l, list):
         raise ValueError("l is not a list.")
-    
+
     if len(l) == 0:
         return None
-    
+
     t = type(l[0])
     for e in l:
-        if type(e) != t:
+        if not isinstance(e, t):
             return None
-    
+
     return t
 
 
@@ -26,20 +27,21 @@ def flatData(data, name, separator = "_"):
     """
     Transforms the parameter value into a flatten dictionary, using the
     parameter name as a prefix for the dictionary keys.
-         
+
     """
     res={}
 
     auxName=name
     if name != "":
         auxName = auxName + separator
-    if type(data) is dict:
+
+    if isinstance(data, dict):
         for key, val in data.items():
             res.update(flatData(val, auxName+str(key), separator))
-    elif hasattr(data, "__iter__"):   
+    elif hasattr(data, "__iter__"):
         for i, val in enumerate(data):
             res.update(flatData(val, auxName+"%d"%i, separator))
     else:
         res = {name:data}
-    
-    return res
\ No newline at end of file
+
+    return res
