@@ -2,7 +2,8 @@
 
 import numpy as np
 import scipy as sp
-from numba import njit, float64,int64
+from sklearn.neighbors import KDTree
+from numba import njit
 
 def bandPower(spectrum, bandsLimits, freqRes, normalize=False):
     """
@@ -17,10 +18,10 @@ def bandPower(spectrum, bandsLimits, freqRes, normalize=False):
             This parameter is used to indicate the bands that are going to be
             used. It is a dict with the name of each band as key and a tuple
             with the lower and upper bounds as value.
-        
+
         freqRes: float
             Minimum resolution for the frequency.
-        
+
         normalize: bool, optional
             If True the each band power is divided by the total power of the
             spectrum. Default False.
@@ -31,20 +32,20 @@ def bandPower(spectrum, bandsLimits, freqRes, normalize=False):
             The keys are the name of each band and the values are their power.
         """
     total = 1
-    if(normalize):
-        total = sp.integrate.simps(spectrum, dx=freqRes)
-    return {key:sp.integrate.simps(spectrum[band[0]:band[1]], dx=freqRes)/total
+    if normalize:
+        total = sp.integrate.trapz(spectrum, dx=freqRes)
+    return {key:sp.integrate.trapz(spectrum[band[0]:band[1]], dx=freqRes)/total
             for key, band in bandsLimits.items()}
 
 def PFD(data):
     """
     Returns the Petrosian Fractal Dimension of the signal given in data.
-    
+
     Parameters
     ----------
     data: array_like
         Signal
-    
+
     Returns
     -------
     float
@@ -52,7 +53,9 @@ def PFD(data):
     """
     derivative = np.diff(data)
     size=len(data)
-    return np.log(size) / (np.log(size) + np.log(size / (size + 0.4 * countSignChanges(derivative))))
+    signChanges = countSignChanges(derivative)
+    logSize = np.log(size)
+    return logSize / (logSize + np.log(size / (size + 0.4 * signChanges)))
 
 def HFD(data,kMax=None):
     """
@@ -71,24 +74,24 @@ def HFD(data,kMax=None):
     float
         The resulting value
     """
-    #Inicializations  
+    #Inicializations
     data=np.array(data)
-    N = len(data)  
+    N = len(data)
     kMax = N // 4 if kMax is None else kMax     #Default kMax
     L = np.zeros(kMax-1)
     x = np.array((-np.log(np.arange(2,kMax+1)),np.ones(kMax-1))).transpose()
-    
+
     return _HFD(data, N, kMax, L, x)
-    
-    
+
+
 @njit
-def _HFD(data, N, kMax, L, x):
+def _HFD(data, N, kMax, L, x):# pragma: no cover
     # Loop from 2 to kMax
     for k in range(2, kMax + 1):
         Lk = np.zeros(k)
         #Loop for compute the lenght of Lmk
         for m in range(0, k):
-            Lmk = 0            
+            Lmk = 0
             for i in range(1, (N - m) // k):
                 Lmk += abs(data[m + i * k] - data[m + i * k - k])
             Lk[m]=Lmk * (N - 1) / (((N - m) // k) * k * k)
@@ -131,29 +134,29 @@ def synchronizationLikelihood(c1, c2, m, l, w1, w2, pRef=0.05, epsilonIterations
         at all and 1 means that they are totally synchronized.
     """
     if len(c1)!=len(c2):
-        raise ValueError("c1 and c2 must have the same lenght")        
-    
+        raise ValueError("c1 and c2 must have the same lenght")
+
     c1 = np.array(c1)
     c2 = np.array(c2)
-    
-    return __SL(c1, c2, m, l, w1, w2, pRef,epsilonIterations)
+
+    return _SL(c1, c2, m, l, w1, w2, pRef,epsilonIterations)
 
 # Auxiliar functions for Synchronization Likeihood
 @njit
-def __SL(c1, c2, m, l, w1, w2, pRef, epsilonIterations):
-    X1 = __getEmbeddedVectors(c1, m, l)
-    X2 = __getEmbeddedVectors(c2, m, l)
-    
-    D1 = __getDistances(X1)
-    D2 = __getDistances(X2)
-    
+def _SL(c1, c2, m, l, w1, w2, pRef, epsilonIterations):# pragma: no cover
+    X1 = _getEmbeddedVectors(c1, m, l)
+    X2 = _getEmbeddedVectors(c2, m, l)
+
+    D1 = _getDistances(X1)
+    D2 = _getDistances(X2)
+
     size=len(X1)
-    E1 = np.zeros(size) 
+    E1 = np.zeros(size)
     E2 = np.zeros(size)
     for i in range(size):
-        E1[i]=__getEpsilon(D1, i, pRef,epsilonIterations)
-        E2[i]=__getEpsilon(D2, i, pRef,epsilonIterations)
-    
+        E1[i]=_getEpsilon(D1, i, pRef,epsilonIterations)
+        E2[i]=_getEpsilon(D2, i, pRef,epsilonIterations)
+
     SL = 0
     SLMax = 0
     for i in range(size):
@@ -170,7 +173,7 @@ def __SL(c1, c2, m, l, w1, w2, pRef, epsilonIterations):
     return SL / SLMax if SLMax>0 else 0
 
 @njit
-def __getHij(D, i, e):
+def _getHij(D, i, e):# pragma: no cover
     summ = 0
     for j in range(len(D)):
         if D[i,j] < e:
@@ -178,7 +181,7 @@ def __getHij(D, i, e):
     return summ
 
 @njit
-def __getDistances(X):
+def _getDistances(X):# pragma: no cover
     t=len(X)
     D=np.zeros((t,t),dtype=np.float64)
     for i in range(t):
@@ -187,34 +190,35 @@ def __getDistances(X):
 
     return D
 @njit
-def __getProbabilityP(D, i, e):
-    return __getHij(D, i, e) /len(D) 
+def _getProbabilityP(D, i, e):# pragma: no cover
+    return _getHij(D, i, e) /len(D)
 
 @njit
-def __getEmbeddedVectors(x, m, l):
+def _getEmbeddedVectors(x, m, l):# pragma: no cover
     size = len(x) - (m - 1) * l
-    X = np.zeros((size,m))
-    for i in range(size):
-        X[i]=x[i:i + m * l:l]
+    X = np.zeros((m, size))
+    for i in range(m):
+        X[i]=x[i*l:i * l + size]
 
-    return X
+    return X.T
 
 @njit
-def __logDiference(p1,p2):
+def _logDiference(p1,p2):# pragma: no cover
     return abs(np.log(p2/p1))
 
 @njit
-def __getEpsilon(D, i, pRef, iterations):
+def _getEpsilon(D, i, pRef, iterations):# pragma: no cover
     eInf = 0
     eSup = None
     bestE=e = 1
     bestP=p = 1
     minP = 1 / len(D)
     for _ in range(iterations):
-        p = __getProbabilityP(D, i, e)
+        p = _getProbabilityP(D, i, e)
         if pRef < minP == p:
             break
-        elif p < pRef:
+        
+        if p < pRef:
             eInf = e
         elif p > pRef:
             eSup = e
@@ -222,11 +226,12 @@ def __getEpsilon(D, i, pRef, iterations):
             bestP=p
             bestE=e
             break
-        if __logDiference(bestP,pRef) > __logDiference(p,pRef):
+        
+        if _logDiference(bestP,pRef) > _logDiference(p,pRef):
             bestP=p
             bestE=e
         e = e * 2 if eSup is None else (eInf + eSup) / 2
-        
+
     return bestE
 
 
@@ -301,14 +306,14 @@ def hjorthComplexity(data):
 
 
 # Sample Entropy
-def MSE(data, m = 2, l = 1, r = None, fr = 0.2, eps = 1e-10):
+def sampEn(data, m = 2, l = 1, r = None, fr = 0.2, eps = 1e-10):
     """
-    Returns Multiscale Sample Entropy of the given data.
-    
+    Returns Sample Entropy of the given data.
+
     Parameters
     ----------
     data: array_like
-        The signal  
+        The signal
     m: int, optional
         Size of the embedded vectors. By default 2.
     l: int, optional
@@ -321,87 +326,90 @@ def MSE(data, m = 2, l = 1, r = None, fr = 0.2, eps = 1e-10):
     eps: float, optional
         Small number added to avoid infinite results. If 0 infinite results can
         appear. Default: 1e-10.
-    
+
     Returns
     -------
     float
-        The resulting value    
+        The resulting value
     """
     if not r:
         r = fr * np.std(data)
-    
-    A = __countEmbeddedDistances(data, m+1, l, r) + eps
-    B = __countEmbeddedDistances(data, m  , l, r) + eps
-    
-    return -np.log(A/B) if A != 0 else float("inf")
 
-def __countEmbeddedDistances(data, m, l, r):
-    X = __getEmbeddedVectors(data , m, l)
-    
-    #Generator of the chebyshev distances of each pair i,j
-    D = ( np.max(np.abs(X[i+1:]-X[i]),axis=1) for i in range(len(X)-1) )
-    
-    return np.sum(np.fromiter((np.sum(d < r) for d in D), dtype=np.int))
+    A = _countEmbeddedDistances(data, m+1, l, r) + eps
+    B = _countEmbeddedDistances(data, m  , l, r) + eps
+
+    if B == 0:# pragma: no cover
+        return -np.inf
+
+    if A == 0:# pragma: no cover
+        return np.inf
+    return -np.log(A/B)
+
+def _countEmbeddedDistances(data, m, l, r):
+    X = _getEmbeddedVectors(data , m, l)
+
+    kdtree = KDTree(X, metric="chebyshev")
+    # Return the count
+    return np.sum(kdtree.query_radius(X, r, count_only=True) - 1)
 
 
 # Lempel-Ziv Complexity
 def LZC(data, threshold = None):
     """
     Returns the Lempel-Ziv Complexity (LZ76) of the given data.
-    
+
     Parameters
     ----------
     data: array_like
         The signal.
     theshold: numeric, optional
         A number use to binarize the signal. The values of the signal above
-        threshold will be converted to 1 and the rest to 0. By default, the 
+        threshold will be converted to 1 and the rest to 0. By default, the
         median of the data.
-        
+
     References
     ----------
-    .. [1] M. Aboy, R. Hornero, D. Abasolo and D. Alvarez, "Interpretation of 
-           the Lempel-Ziv Complexity Measure in the Context of Biomedical 
-           Signal Analysis," in IEEE Transactions on Biomedical Engineering, 
+    .. [1] M. Aboy, R. Hornero, D. Abasolo and D. Alvarez, "Interpretation of
+           the Lempel-Ziv Complexity Measure in the Context of Biomedical
+           Signal Analysis," in IEEE Transactions on Biomedical Engineering,
            vol. 53, no.11, pp. 2282-2288, Nov. 2006.
     """
     if not threshold:
         threshold=np.median(data)
-        
+
     n = len(data)
-    
-    sequence = __binarize(data, threshold)
-    
-    c = __LZC(sequence)
+
+    sequence = _binarize(data, threshold)
+
+    c = _LZC(sequence)
     b = n/np.log2(n)
-    
-    
+
     lzc = c/b
-    
+
     return lzc
 
 @njit
-def __LZC(sequence):
+def _LZC(sequence):# pragma: no cover
     n = len(sequence)
     complexity = 1
-    
+
     q0    = 1
     qSize = 1
-    
+
     sqi   = 0
     where = 0
-    
-    while(q0 + qSize <= n):
+
+    while q0 + qSize <= n:
         # If we are checking the end of the sequence we just need to look at
         # the last element
-        if not sqi == q0-1:
-            contained, where = __isSubsequenceContained(sequence[q0:q0+qSize],
+        if sqi != q0-1:
+            contained, where = _isSubsequenceContained(sequence[q0:q0+qSize],
                                                     sequence[sqi:q0+qSize-1])
         else:
             contained = sequence[q0+qSize] == sequence[q0+qSize-1]
-        
+
          #If Q is contained in sq~, we increase the size of q
-        if(contained):
+        if contained:
             qSize+=1
             sqi = where
         #If Q is not contained the complexity is increased by 1 and reset Q
@@ -410,19 +418,18 @@ def __LZC(sequence):
             qSize=1
             complexity+=1
             sqi=0
-    
-    
+
     return complexity
 
 
-def __binarize(data, threshold):
-    if type(data) != np.ndarray:
+def _binarize(data, threshold):
+    if  not isinstance(data, np.ndarray):
         data = np.array(data)
-    
+
     return np.array(data > threshold, np.uint8)
 
 @njit
-def __isSubsequenceContained(subSequence, sequence):
+def _isSubsequenceContained(subSequence, sequence):# pragma: no cover
     """
     Checks if the subSequence is into the sequence and returns a tuple that
     informs if the subsequence is into and where. Return examples: (True, 7),
@@ -430,18 +437,17 @@ def __isSubsequenceContained(subSequence, sequence):
     """
     n = len(sequence)
     m = len(subSequence)
-    
+
     for i in range(n-m+1):
         equal = True
         for j in range(m):
             equal = subSequence[j] == sequence[i+j]
             if not equal:
                 break
-        
+
         if equal:
             return True, i
-        
-    
+
     return False, -1
 
 
@@ -450,7 +456,7 @@ def DFA(data, fit_degree = 1, min_window_size = 4, max_window_size = None,
         fskip = 1, max_n_windows_sizes=None):
     """
     Applies Detrended Fluctuation Analysis algorithm to the given data.
-    
+
     Parameters
     ----------
     data: array_like
@@ -468,7 +474,7 @@ def DFA(data, fit_degree = 1, min_window_size = 4, max_window_size = None,
         Maximum number of window sizes that will be used. The final number can
         be smaller once the repeated values are removed
         Default: log2(size)
-    
+
     Returns
     -------
     float
@@ -476,69 +482,43 @@ def DFA(data, fit_degree = 1, min_window_size = 4, max_window_size = None,
     """
     #Arguments handling
     data = np.array(data)
-    
+
     size=len(data)
-    
+
     if not max_window_size:
         max_window_size = size//4
-    
-    #Detrended data    
+
+    #Detrended data
     Y = np.cumsum(data - np.mean(data))
-    
+
     #Windows sizes
     if not max_n_windows_sizes:
         max_n_windows_sizes = int(np.round(np.log2(size)))
-        
+
     ns = np.unique(
           np.geomspace(min_window_size, max_window_size, max_n_windows_sizes,
                        dtype=int))
-    
+
     #Fluctuations for each window size
     F = np.zeros(ns.size)
-    
+
     #Loop for each window size
     for indexF,n in enumerate(ns):
         itskip = max(int(fskip * n),1)
         nWindows = int(np.ceil((size - n + 1) / itskip))
-        
+
         #Aux x
         x = np.arange(n)
-        
+
         y  = np.array([Y[i*itskip:i*itskip+n] for i in range(0,nWindows)])
         c  = np.polynomial.polynomial.polyfit(x, y.T, fit_degree)
         yn = np.polynomial.polynomial.polyval(x, c)
-            
-        F[indexF] = np.mean(np.sqrt(np.sum((y-yn)**2, axis=1)/n))
-    
-    alpha = np.polyfit(np.log(ns), np.log(F), 1)[0]
-    
-    if np.isnan(alpha):
-        return 0
-    
-    return alpha
 
-# Cross Correlation Coeficient
-def CCC(c1, c2):
-    """
-    Computes the Cross Correlation Coeficient between the data in c1 and the 
-    data in c2.
-    
-    Parameters
-    ----------
-    c1: array_like
-        One of the signals
-    c2: array_like
-        The other signal
-    
-    Returns
-    -------
-    float
-        The Cross Correlation Coeficient.
-    """
-    meanC1 =  np.mean(c1)
-    meanC2 =  np.mean(c2)
-    
-    num = np.mean((c1 - meanC1)*(c2-meanC2))
-    den = np.sqrt(np.var(c1)*np.var(c2))
-    
-    return num/den if num != 0 else 0
\ No newline at end of file
+        F[indexF] = np.mean(np.sqrt(np.sum((y-yn)**2, axis=1)/n))
+
+    alpha = np.polyfit(np.log(ns), np.log(F), 1)[0]
+
+    if np.isnan(alpha): # pragma: no cover
+        return 0
+
+    return alpha
