@@ -6,6 +6,8 @@ every feature in each window configured. It also allows specifying labels for
 the whole signal or for specific segments.
 """
 import pandas as pd
+import progressbar
+
 from eeglib.auxFunctions import flatData
 from eeglib.eeg import EEG
 
@@ -97,8 +99,10 @@ class Wrapper():
     once. The main usage of this class if for generating features to use with
     machine learning techniques.
     """
+    _indexName = "window_id"
     def __init__(self, helper, flat = True, flatSeparator = "_", store=True,
-                 label = None, segmentation = None):
+                 label = None, segmentation = None, onlySegments = False,
+                 showProgress = False):
         """
         Parameters
         ----------
@@ -126,6 +130,11 @@ class Wrapper():
             :py:meth:`eeglib.helpers.Helper.moveEEGWindow`. The label of the
             unlabelled segments will be 0. If None, no segmentation will be
             added. Default: None.
+        onlySegments: bool, optional
+            If True, only windows between segments will be computed. Only used
+            if a segmentation argument is used.
+        showProgress: bool, optional
+            If True, it will shown the progress when computing all the features.
         """
         self.helper = helper
         self.iterator = iter(self.helper)
@@ -147,19 +156,63 @@ class Wrapper():
                                  for (s0, se), label in segmentation]
             self.segmentation = sorted(self.segmentation)
             self.segmentationIndex = 0
+            self.onlySegments = onlySegments
+            self.insideSegment = False
         else:
             self.segmentation = None
 
+        self.showProgress = showProgress
         self.addFeature = _FeatureAdder(self)
 
     def __iter__(self):
         self.iterator = iter(self.helper)
+        # If show progress, init progress bar
+        if self.showProgress:
+            step = self.iterator.step
+            w_size = self.helper.windowSize
+            max_progress = (self.iterator.endPoint-w_size)/step+1
+            self.progressBar = progressbar.ProgressBar(max_value=max_progress)
         return self
 
     def __next__(self):
+        # Show progress
+        if self.showProgress:
+            progress = self.iterator.auxPoint/self.iterator.step
+            self.progressBar.update(progress)
+            
+        # Move iterator
         next(self.iterator)
+        
+        # Manage segmentation and segmentation skips
+        if self.segmentation:
+            self._updateSegmentStatus()
+            if self.onlySegments and not self.insideSegment:
+                if self.segmentationIndex >= len(self.segmentation):
+                    raise StopIteration
+                segment = self.segmentation[self.segmentationIndex]
+                self.iterator.auxPoint = segment[0][0]
+                self.insideSegment = True
+                next(self.iterator)
         return self.getFeatures()
 
+    def _updateSegmentStatus(self):
+        self.insideSegment = False
+        # Loop through all segments to find the current one
+        while self.segmentationIndex < len(self.segmentation):
+            curSeg, _ = self.segmentation[self.segmentationIndex]
+            curPoint = self.iterator.auxPoint - self.iterator.step
+            # If current point inside bounds, segment found
+            if curSeg[0] <= curPoint < curSeg[1]:
+                self.insideSegment = True
+                break
+            # If current point is before the segment begin, we are out
+            # the segment
+            elif curPoint <curSeg[0]:
+                break
+            else:
+                self.segmentationIndex+=1
+                
+
     def addFeatures(self, features):
         """
         Parameters
@@ -177,14 +230,16 @@ class Wrapper():
                 raise ValueError("Only tuples of len 3 or str are valid values")
 
     def addCustomFeature(self, function, channels = None, twoChannels = False,
-                         name = None):
+                         name = None, customArgs = [], customKwargs = {}):
         """
         Adds a custom feature that will be included in the dataset.
 
         Parameters
         ----------
         function: function
-            The function to apply.
+            The function to apply. It must take an array-like as first parameter 
+            if twoChannels parameter is False or two array-like for each of the
+            first two parameters if twoChannels is True.
 
         channels: Variable type, optional
             The channels over which the function will be applied.
@@ -198,22 +253,34 @@ class Wrapper():
         twoChannels: bool
             If function receives two channels of data this parameter should be
             True. Default: False.
+        
         name: str
             A custom name for the feature that will be visible in the df.
+        
+        customArgs: list
+            A list of fixed arguments for the function.
+        
+        customKwargs: dict
+            A dict of keyword arguments, where the key is the argument name and
+            the value is the argument content.
 
         Returns
         -------
         None
         """
 
-        if not twoChannels:
-            f = lambda: self.helper.eeg._applyFunctionTo(function, channels)
-        else:
-            f = lambda: self.helper.eeg._applyFunctionTo2C(function, channels)
-
         if not name:
             name = function.__name__
 
+        if not twoChannels:
+            aux_func = lambda x: function(x, *customArgs, **customKwargs)
+            f = lambda: self.helper.eeg._applyFunctionTo(aux_func, channels)
+        else:
+            aux_func = lambda x1, x2: function(x1, x2, *customArgs, **customKwargs)
+            f = lambda: self.helper.eeg._applyFunctionTo2C(aux_func, channels)
+
+        
+
         self.functions[name] = f
 
 
@@ -236,23 +303,19 @@ class Wrapper():
             features = self.storedFeatures[-1]
         else:
             features = {name:f() for name, f in self.functions.items()}
+            next_window = self.iterator.auxPoint
+            step = self.iterator.step
+            features[self._indexName] = int(next_window/step - 1)
             if self.flat:
                 features = flatData(features,"", self.flatSeparator)
 
             if self.segmentation:
-                loop = True
-                features["segment_label"] = 0
-                while loop and self.segmentationIndex < len(self.segmentation):
-                    curSeg, curLab = self.segmentation[self.segmentationIndex]
-                    curPoint = self.iterator.auxPoint - self.iterator.step
-                    if curSeg[0] <= curPoint < curSeg[1]:
-                        features["segment_label"] = curLab
-                        loop = False
-                    elif curPoint <curSeg[0]:
-                        loop = False
-                    else:
-                        self.segmentationIndex+=1
-
+                if self.insideSegment:
+                    segment = self.segmentation[self.segmentationIndex]
+                    features["segment_label"] = segment[1]
+                else:
+                    features["segment_label"] = 0
+            
             if self.label is not None:
                 features["label"] = self.label
 
@@ -268,7 +331,9 @@ class Wrapper():
         None.
         """
         if self.store:
-            return pd.DataFrame(self.storedFeatures)
+            df = pd.DataFrame(self.storedFeatures)
+            df.set_index(self._indexName, inplace=True)
+            return df
         return None
 
     def getAllFeatures(self):
@@ -282,7 +347,11 @@ class Wrapper():
         data=[features for features in self]
         if self.store:
             data=self.storedFeatures
-        return pd.DataFrame(data)
+            
+        df = pd.DataFrame(data)
+        df.set_index(self._indexName, inplace=True)
+        return df
+
 
     def reset(self):
         """
